package gate.creole.ontology.owlim;

import gate.creole.ontology.DataType;
import gate.creole.ontology.OConstants;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;

import org.openrdf.vocabulary.RDF;
import org.openrdf.vocabulary.RDFS;

import com.ibm.icu.text.DateFormat;

/**
 * A class that produces events log compatible with NeonToolKit. For
 * more information on this, please go through the deliverable 3.1.2 of
 * the neon project.
 * 
 * @author niraj aswani
 */
public class NeonOntologyEventsLog implements OConstants {

  /**
   * list of events. list is used to keep track of order in which
   * changes occurred.
   */
  private List<OEvent> events = new ArrayList<OEvent>();

  /**
   * List of changes - used internally by log producer to record
   * dependencies of changes.
   */
  private List<String> changes = new ArrayList<String>();

  /**
   * name of the ontology that has been changed.
   */
  private String changedOntology;

  /**
   * url of the ontology that has been changed.
   */
  private String ontologyURL;

  /**
   * indicates addition something to the ontology.
   */
  public final int ADDITION = 1;

  /**
   * indicates removal of something from the ontology.
   */
  public final int REMOVAL = 2;

  public String ontologyInst = null;

  public boolean enabled = true;
  
  /**
   * various namespaces.
   */
  public static final String a = "http://omv.ontoware.org/2005/05/ontology#";

  public static final String b = "http://omv.ontoware.org/2007/05/pomv#";

  public static final String c = "http://omv.ontoware.org/2007/05/mappingomv#";

  public static final String d = "http://omv.ontoware.org/2007/07/workflow#";

  public static final String e = "http://owlodm.ontoware.org/OWL1.1#";

  public static final String f = "http://omv.ontoware.org/2007/10/changes#";

  /**
   * anyone that uses gate to make modifications is identified as gate
   * user
   */
  public final String USER_URI = "http://www.gate.ac.uk/ontology#gate";

  /**
   * Constructor
   * 
   * @param ontologyName - name of the ontology
   * @param ontologyUrl - url of the ontology
   */
  public NeonOntologyEventsLog(String ontologyName, String ontologyUrl) {
    // first thing first
    // lets create
    changedOntology = ontologyName;
    ontologyURL = ontologyUrl;
    ontologyInst = changedOntology.replaceAll("[^a-zA-Z0-9]+", "_");
    if(!ontologyInst.startsWith("http:") && !ontologyInst.startsWith("file")) {
      ontologyInst = "http://gate.ac.uk/" + ontologyInst;
    }

    // initial setup
    // adding statements to identify ontology and the user who's making
    // changes
    addEvent(ontologyInst, RDF.TYPE, a + "Ontology");
    addEvent(USER_URI, RDF.TYPE, a + "Person");
    addEvent(USER_URI, a + "firstName", "gate", DataType.getStringDataType()
            .getXmlSchemaURI().toString());
    addEvent(USER_URI, a + "lastName", "gate", DataType.getStringDataType()
            .getXmlSchemaURI().toString());
    addEvent("http://gate.ac.uk/;" + ontologyName + ";changeSpecification",
            RDF.TYPE, f + "ChangeSpecification");

    String ontoURL = ontologyURL.toString();
    String urlParent = ontoURL;
    if(urlParent.indexOf('/') > 1) {
      urlParent = ontoURL.substring(0, ontoURL.lastIndexOf('/'));
    }
    String toReturn = ontoURL + "?location=" + ontoURL + ";changeSpecification";
    addEvent("http://gate.ac.uk/;" + ontologyName + ";changeSpecification", f
            + "URI", f + toReturn);
  }

  /**
   * adds a new event to the events list. all of the following must be
   * valid URIs
   * 
   * @param s subject
   * @param p predicate
   * @param o object
   */
  private void addEvent(String s, String p, String o) {
    if(enabled)
      events.add(new OEvent(s, p, o, true));
  }

  /**
   * adds a new event to the events list. all of the following must be
   * valid URIs except the value of o.
   * 
   * @param s subject
   * @param p predicate
   * @param o object/value
   * @param d datatype
   */
  private void addEvent(String s, String p, String o, String d) {
    if(enabled)
      events.add(new OEvent(s, p, o, d, true));
  }

  /**
   * returns current time stamp
   * 
   * @return
   */
  public static String getTimeStamp() {
    return DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM,
            Locale.US).format(Calendar.getInstance().getTime());
  }

  /**
   * create a random uri. format:
   * ontologyURL?location=urlbase;ontologyName;keyWord=randomString
   * 
   * @param ontologyURL
   * @param ontologyName
   * @param keyWord
   * @return
   */
  public static String createRandomURI(String ontologyURL, String ontologyName,
          String keyWord) {
    String ontoURL = ontologyURL.toString();
    String urlParent = ontoURL;
    if(urlParent.indexOf('/') > 1) {
      urlParent = ontoURL.substring(0, ontoURL.lastIndexOf('/'));
    }
    String toReturn = ontoURL + "?location=" + urlParent;
    toReturn += ";" + ontologyName.replaceAll("[^a-zA-Z0-9]+", "_") + ";"
            + keyWord + "="
            + ("" + Math.random()).replaceAll("[^a-zA-Z0-9]+", "_");
    return toReturn;
  }

  /**
   * Adds a declaration of entity by adding three statements <BR>
   * axionRandomURI, type, Declaration <BR> axiomRandomURI, entity, entity
   * <BR> atomicChange, appliedAxiom, axiomRandomURI
   * 
   * @param entity
   * @param atomicChange
   * @return axiomRandomURI
   */
  public String declaration(String entity, String atomicChange) {
    String declaration = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(declaration, RDF.TYPE, e + "Declaration");
    addEvent(declaration, e + "entity", entity);
    addEvent(atomicChange, f + "appliedAxiom", declaration);
    return declaration;
  }

  /**
   * registers change to indicate the affected entity 
   * <BR> changeInstURI, appliedToOntology, ontologyInst <BR> changedInstURI,
   * consistsOfAtomicOperation, lastRegisteredChange <BR> changedInstURI,
   * hasRelatedEntity, relatedEntity <BR> changedInstURI, hasAuthor, user
   * <BR> changedInstURI, timestamp, getTimeStamp
   * 
   * @param changeInstURI
   * @param relatedEntityURI
   */
  public void registerChange(String changeInstURI, String relatedEntityURI) {
    addEvent(changeInstURI, f + "appliedToOntology", ontologyInst);
    addEvent(changeInstURI, f + "consistsOfAtomicOperation", changes
            .get(changes.size() - 1));
    addEvent(changeInstURI, f + "hasRelatedEntity", relatedEntityURI);
    addEvent(changeInstURI, f + "hasAuthor", USER_URI);
    addEvent(changeInstURI, d + "timestamp", getTimeStamp(), DataType
            .getStringDataType().getXmlSchemaURI().toString());
  }

  /**
   * registers the atomic change
   * <BR> atomicChange, type, Addition
   * <BR> atomicChange, timestamp, getTimeStamp()
   * <BR> atomicChange, appliedToOntology, ontologyInst
   * <BR> atomicChange, hasAuthor, username
   * @param changeType - ADDITION or REMOVAL
   * @return atomicChangeRandomURI
   */
  public String atomicChange(int changeType) {
    String atomicChange = createRandomURI(ontologyURL, changedOntology,
            "change");
    
    // tell the log if it is an instance of Addition or Removal
    if(changeType == ADDITION) {
      addEvent(atomicChange, RDF.TYPE, f + "Addition");
    }
    else {
      addEvent(atomicChange, RDF.TYPE, f + "Removal");
    }

    // when it occurred
    addEvent(atomicChange, d + "timestamp", getTimeStamp(), DataType
            .getStringDataType().getXmlSchemaURI().toString());
    addEvent(atomicChange, f + "appliedToOntology", ontologyInst);
    addEvent(atomicChange, f + "hasAuthor", USER_URI);

    if(changes.size() > 0) {
      addEvent(atomicChange, f + "hasPreviousChange", changes.get(changes
              .size() - 1));
    }
    addChange(atomicChange);
    return atomicChange;
  }

  /**
   * registers addition or removal of the resource
   * @param r
   * @param changeClass
   * @param addition
   */
  private void addRemoveResource(String r, String changeClass, boolean addition) {
    String atomicChange = atomicChange(addition ? ADDITION : REMOVAL);
    declaration(r, atomicChange);
    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, changeClass);
    registerChange(changeURI, r);
  }

  /**
   * registers removal of the class
   * @param r
   * @param addition
   */
  public void addRemoveOClass(String r, boolean addition) {
    addRemoveResource(r, f + (addition ? "AddClass" : "RemoveClass"), addition);
  }

  /**
   * Add remove object property
   * @param r
   * @param addition
   */
  public void addRemoveObjectProperty(String r, boolean addition) {
    addRemoveResource(r, f
            + (addition ? "AddObjectProperty" : "RemoveObjectProperty"),
            addition);
  }

  public void addRemoveDatatypeProperty(String r, boolean addition) {
    addRemoveResource(r, f
            + (addition ? "AddDataProperty" : "RemoveDataProperty"), addition);
  }

  public void addRemoveDatatype(String d, boolean addition) {
    addRemoveResource(d, f + (addition ? "AddDatatype" : "RemoveDatatype"),
            addition);
  }

  public void addRemoveAnnotationProperty(String a, boolean addition) {
    addRemoveResource(d,
            f
                    + (addition
                            ? "AddAnnotationProperty"
                            : "RemoveAnnotationProperty"), addition);
  }

  public void addProperty(String a, boolean addition) {
    addRemoveResource(d, f + (addition ? "AddProperty" : "RemoveProperty"),
            addition);
  }

  
  public void addRemoveDisjointClasses(String c1, String c2, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "DisjointClasses");
    addEvent(dc, e + "disjointClasses", c1);
    addEvent(dc, e + "disjointClasses", c2);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddDisjointClass" : "RemoveDisjointClass"));
    registerChange(changeURI, c1);
  }

  public void addRemoveSubClassOf(String subClass, String superClass,
          boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubClassOf");
    addEvent(dc, e + "subClass", subClass);
    addEvent(dc, e + "superClass", superClass);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddSubClassOf" : "RemoveSubClassOf"));
    registerChange(changeURI, subClass);
  }

  //TODO add entries for the following methods in OWLIMServiceImpl
  public void addRemoveAllValuesFrom(String subClass, String restriction,
          String onProperty, String range, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubClassOf");
    addEvent(dc, e + "subClass", subClass);

    String dc1 = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc1, RDF.TYPE, e + "DataAllValuesFrom");
    addEvent(dc1, e + "dataProperties", onProperty);
    addEvent(dc1, e + "dataRange", range);

    addEvent(dc, e + "superClass", dc1);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddSubClassOf" : "RemoveSubClassOf"));
    registerChange(changeURI, subClass);
  }

  public void addRemoveObjectAllValuesFrom(String subClass, String restriction,
          String onProperty, String owlClass, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubClassOf");
    addEvent(dc, e + "subClass", subClass);

    String dc1 = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc1, RDF.TYPE, e + "ObjectAllValuesFrom");
    addEvent(dc1, e + "objectProperty", onProperty);
    addEvent(dc1, e + "class", owlClass);

    addEvent(dc, e + "superClass", dc1);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddSubClassOf" : "RemoveSubClassOf"));
    registerChange(changeURI, subClass);
  }

  public void addRemoveDataSomeValuesFrom(String subClass, String restriction,
          String onProperty, String range, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubClassOf");
    addEvent(dc, e + "subClass", subClass);

    String dc1 = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc1, RDF.TYPE, e + "DataSomeValuesFrom");
    addEvent(dc1, e + "dataProperties", onProperty);
    addEvent(dc1, e + "dataRange", range);

    addEvent(dc, e + "superClass", dc1);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddSubClassOf" : "RemoveSubClassOf"));
    registerChange(changeURI, subClass);
  }

  public void addRemoveObjectSomeValuesFrom(String subClass,
          String restriction, String onProperty, String owlClass,
          boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubClassOf");
    addEvent(dc, e + "subClass", subClass);

    String dc1 = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc1, RDF.TYPE, e + "ObjectSomeValuesFrom");
    addEvent(dc1, e + "objectProperty", onProperty);
    addEvent(dc1, e + "class", owlClass);

    addEvent(dc, e + "superClass", dc1);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddSubClassOf" : "RemoveSubClassOf"));
    registerChange(changeURI, subClass);
  }

  public void addRemoveDataMinCardinality(String subClass, String restriction,
          String onProperty, String range, String cardinality, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubClassOf");
    addEvent(dc, e + "subClass", subClass);

    String dc1 = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc1, RDF.TYPE, e + "DataMinCardinality");
    addEvent(dc1, e + "dataProperty", onProperty);
    addEvent(dc1, e + "dataRange", range);
    addEvent(dc1, e + "cardinality", cardinality, DataType.getIntDataType()
            .getXmlSchemaURI().toString());

    addEvent(dc, e + "superClass", dc1);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddSubClassOf" : "RemoveSubClassOf"));
    registerChange(changeURI, subClass);
  }

  public void addRemoveObjectMinCardinality(String subClass,
          String restriction, String onProperty, String owlClass,
          String cardinality, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubClassOf");
    addEvent(dc, e + "subClass", subClass);

    String dc1 = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc1, RDF.TYPE, e + "ObjectMinCardinality");
    addEvent(dc1, e + "objectProperty", onProperty);
    addEvent(dc1, e + "class", owlClass);
    addEvent(dc1, e + "cardinality", cardinality, DataType.getIntDataType()
            .getXmlSchemaURI().toString());

    addEvent(dc, e + "superClass", dc1);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddSubClassOf" : "RemoveSubClassOf"));
    registerChange(changeURI, subClass);
  }

  public void addRemoveDataMaxCardinality(String subClass, String restriction,
          String onProperty, String range, String cardinality, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubClassOf");
    addEvent(dc, e + "subClass", subClass);

    String dc1 = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc1, RDF.TYPE, e + "DataMaxCardinality");
    addEvent(dc1, e + "dataProperty", onProperty);
    addEvent(dc1, e + "dataRange", range);
    addEvent(dc1, e + "cardinality", cardinality, DataType.getIntDataType()
            .getXmlSchemaURI().toString());

    addEvent(dc, e + "superClass", dc1);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddSubClassOf" : "RemoveSubClassOf"));
    registerChange(changeURI, subClass);
  }

  public void addRemoveObjectMaxCardinality(String subClass,
          String restriction, String onProperty, String owlClass,
          String cardinality, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubClassOf");
    addEvent(dc, e + "subClass", subClass);

    String dc1 = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc1, RDF.TYPE, e + "ObjectMaxCardinality");
    addEvent(dc1, e + "objectProperty", onProperty);
    addEvent(dc1, e + "class", owlClass);
    addEvent(dc1, e + "cardinality", cardinality, DataType.getIntDataType()
            .getXmlSchemaURI().toString());

    addEvent(dc, e + "superClass", dc1);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddSubClassOf" : "RemoveSubClassOf"));
    registerChange(changeURI, subClass);
  }

  public void addRemoveDataExactCardinality(String subClass,
          String restriction, String onProperty, String range,
          String cardinality, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubClassOf");
    addEvent(dc, e + "subClass", subClass);

    String dc1 = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc1, RDF.TYPE, e + "DataExactCardinality");
    addEvent(dc1, e + "dataProperty", onProperty);
    addEvent(dc1, e + "dataRange", range);
    addEvent(dc1, e + "cardinality", cardinality, DataType.getIntDataType()
            .getXmlSchemaURI().toString());

    addEvent(dc, e + "superClass", dc1);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddSubClassOf" : "RemoveSubClassOf"));
    registerChange(changeURI, subClass);
  }

  public void addRemoveObjectExactCardinality(String subClass,
          String restriction, String onProperty, String owlClass,
          String cardinality, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubClassOf");
    addEvent(dc, e + "subClass", subClass);

    String dc1 = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc1, RDF.TYPE, e + "ObjectExactCardinality");
    addEvent(dc1, e + "objectProperty", onProperty);
    addEvent(dc1, e + "class", owlClass);
    addEvent(dc1, e + "cardinality", cardinality, DataType.getIntDataType()
            .getXmlSchemaURI().toString());

    addEvent(dc, e + "superClass", dc1);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddSubClassOf" : "RemoveSubClassOf"));
    registerChange(changeURI, subClass);
  }
  //TODO end of restrictions
  
  
  public void addRemoveEquivalentClasses(String c1, String c2, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "EquivalentClasses");
    addEvent(dc, e + "equivalentClasses", c1);
    addEvent(dc, e + "equivalentClasses", c2);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddEquivalentClass" : "RemoveEquivalentClass"));
    registerChange(changeURI, c1);
  }

  //TODO add entry for the following in OWLIMServiceImpl
  public void addRemoveDisjointUnion(String c1, String c2, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);

    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "DisjointUnion");
    addEvent(dc, e + "unionClass", c1);
    addEvent(dc, e + "disjointClasses", c2);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    addEvent(changeURI, RDF.TYPE, f
            + (addition ? "AddDisjointUnion" : "RemoveDisjointUnion"));
    registerChange(changeURI, c1);
  }

  public void addRemoveAnnotationPropertyValue(String annotProperty,
          String resource, String value, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "EntityAnnotation");
    addEvent(dc, e + "entity", resource);
    addEvent(dc, e + "annotationProperty", annotProperty);
    addEvent(dc, e + "entityAnnotation", value, DataType.getStringDataType()
            .getXmlSchemaURI().toString());
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(annotProperty.equalsIgnoreCase(RDFS.LABEL)) {
      addEvent(changeURI, RDF.TYPE, f + (addition ? "AddLabel" : "RemoveLabel"));
    }
    else {
      addEvent(changeURI, RDF.TYPE, f
              + (addition ? "AddComment" : "RemoveComment"));
    }
    registerChange(changeURI, resource);
  }

  public void addRemoveObjectPropertyDomain(String objectProperty,
          String domain, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "ObjectPropertyDomain");
    addEvent(dc, e + "objectProperty", objectProperty);
    addEvent(dc, e + "domain", domain);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddObjectPropertyDomain");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveObjectPropertyDomain");
    }
    registerChange(changeURI, objectProperty);
  }

  public void addRemoveObjectPropertyRange(String objectProperty, String range,
          boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "ObjectPropertyRange");
    addEvent(dc, e + "objectProperty", objectProperty);
    addEvent(dc, e + "range", range);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddObjectPropertyRange");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveObjectPropertyRange");
    }
    registerChange(changeURI, objectProperty);
  }

  public void addRemoveSubObjectPropertyOf(String subObjectProperty,
          String superObjectProperty, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubObjectPropertyOf");
    addEvent(dc, e + "subObjectProperties", subObjectProperty);
    addEvent(dc, e + "superObjectProperty", superObjectProperty);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddSubObjectPropertyOf");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveSubObjectPropertyOf");
    }
    registerChange(changeURI, subObjectProperty);
  }

  public void addRemoveEquivalentObjectProperties(String property1,
          String property2, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "EquivalentObjectProperties");
    addEvent(dc, e + "equivalentObjectProperties", property1);
    addEvent(dc, e + "equivalentObjectProperties", property2);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddEquivalentObjectProperty");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveEquivalentObjectProperty");
    }
    registerChange(changeURI, property2);
  }

  public void addRemoveFunctionalObjectProperty(String objectProperty,
          boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "FunctionalObjectProperty");
    addEvent(dc, e + "objectProperty", objectProperty);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddFunctionalObjectProperty");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveFunctionalObjectProperty");
    }
    registerChange(changeURI, objectProperty);
  }

  public void addRemoveOInstance(String instance, String owlClass,
          boolean addition) {
    addRemoveResource(instance, f
            + (addition ? "AddIndividual" : "RemoveIndividual"), addition);
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "ClassAssertion");
    addEvent(dc, e + "individual", instance);
    addEvent(dc, e + "class", owlClass);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddIndividual");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveIndividual");
    }
    registerChange(changeURI, owlClass);
  }

  public void addRemoveInverseFunctionalObjectProperty(String objectProperty,
          boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "InverseFunctionalObjectProperty");
    addEvent(dc, e + "objectProperty", objectProperty);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddInverseFunctionalObjectProperty");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveInverseFunctionalObjectProperty");
    }
    registerChange(changeURI, objectProperty);
  }

  public void addRemoveTransitiveObjectProperty(String objectProperty,
          boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "TransitiveObjectProperty");
    addEvent(dc, e + "objectProperty", objectProperty);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddTransitiveObjectProperty");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveTransitiveObjectProperty");
    }
    registerChange(changeURI, objectProperty);
  }

  public void addRemoveSymmetricObjectProperty(String objectProperty,
          boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SymmetricObjectProperty");
    addEvent(dc, e + "objectProperty", objectProperty);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddSymmetricObjectProperty");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveSymmetricObjectProperty");
    }
    registerChange(changeURI, objectProperty);
  }

  public void addRemoveInverseObjectProperties(String property1,
          String property2, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "InverseObjectProperties");
    addEvent(dc, e + "inverseObjectProperties", property1);
    addEvent(dc, e + "inverseObjectProperties", property2);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddInverseObjectProperties");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveInverseObjectProperties");
    }
    registerChange(changeURI, property2);
  }

  public void addRemoveSameIndividual(String ind1, String ind2, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SameIndividual");
    addEvent(dc, e + "sameIndividuals", ind1);
    addEvent(dc, e + "sameIndividuals", ind2);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddSameIndividual");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveSameIndividual");
    }
    registerChange(changeURI, ind2);
  }

  public void addRemoveDifferentIndividuals(String ind1, String ind2,
          boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "DifferentIndividuals");
    addEvent(dc, e + "differentIndividuals", ind1);
    addEvent(dc, e + "differentIndividuals", ind2);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddDifferentIndividual");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveDifferentIndividual");
    }
    registerChange(changeURI, ind2);
  }

  //TODO we don't support this
  public void addRemoveAsymmetricObjectProperty(String objectProperty,
          boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "AsymmetricObjectProperty");
    addEvent(dc, e + "objectProperty", objectProperty);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddAsymmetricObjectProperty");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveAsymmetricObjectProperty");
    }
    registerChange(changeURI, objectProperty);
  }

  public void addRemoveDisjointObjectProperties(String property1,
          String property2, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "DisjointObjectProperties");
    addEvent(dc, e + "disjointObjectProperties", property1);
    addEvent(dc, e + "disjointObjectProperties", property2);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddDisjointObjectProperties");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveDisjointObjectProperties");
    }
    registerChange(changeURI, property2);
  }

  public void addRemoveDatatypePropertyDomain(String datatypeProperty,
          String domain, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "DataPropertyDomain");
    addEvent(dc, e + "datatypeProperty", datatypeProperty);
    addEvent(dc, e + "domain", domain);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddDataPropertyDomain");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveDataPropertyDomain");
    }
    registerChange(changeURI, datatypeProperty);
  }

  public void addRemoveDataPropertyRange(String datatypeProperty, String range,
          boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "DataPropertyRange");
    addEvent(dc, e + "datatypeProperty", datatypeProperty);
    addEvent(dc, e + "range", range);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddDataPropertyRange");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveDataPropertyRange");
    }
    registerChange(changeURI, datatypeProperty);
  }

  public void addRemoveEquivalentDataProperties(String property1,
          String property2, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "EquivalentDataProperties");
    addEvent(dc, e + "dataProperties", property1);
    addEvent(dc, e + "dataProperties", property2);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddEquivalentDataProperties");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveEquivalentDataProperties");
    }
    registerChange(changeURI, property2);
  }

  public void addRemoveSubDataPropertyOf(String subDataProperty,
          String superDataProperty, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "SubDataPropertyOf");
    addEvent(dc, e + "subDataProperties", subDataProperty);
    addEvent(dc, e + "superDataProperty", superDataProperty);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddSubDataPropertyOf");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveSubDataPropertyOf");
    }
    registerChange(changeURI, subDataProperty);
  }

  public void addRemoveFunctionalDataProperty(String dataProperty,
          boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "FunctionalDataProperty");
    addEvent(dc, e + "dataProperty", dataProperty);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddFunctionalDataProperty");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveFunctionalDataProperty");
    }
    registerChange(changeURI, dataProperty);
  }

  public void addRemoveDisjointDataProperties(String property1,
          String property2, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "DisjointDataProperties");
    addEvent(dc, e + "disjointDataProperties", property1);
    addEvent(dc, e + "disjointDataProperties", property2);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddDisjointDataProperties");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveDisjointDataProperties");
    }
    registerChange(changeURI, property2);
  }

  public void addRemoveDatatypePropertyValue(String datatypeProperty,
          String individual, String value, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "DataPropertyAssertion");
    addEvent(dc, e + "sourceIndividual", individual);
    addEvent(dc, e + "dataProperty", datatypeProperty);
    addEvent(dc, e + "targetValue", value, DataType.getStringDataType()
            .getXmlSchemaURI().toString());
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);
    

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddIndividualDataProperty");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveIndividualDataProperty");
    }
    registerChange(changeURI, individual);
  }

  public void addRemoveObjectPropertyValue(String objectProperty,
          String srcIndividual, String tgtIndividual, boolean addition) {
    atomicChange(addition ? ADDITION : REMOVAL);
    String dc = createRandomURI(ontologyURL, changedOntology, "axiom");
    addEvent(dc, RDF.TYPE, e + "ObjectPropertyAssertion");
    addEvent(dc, e + "sourceIndividual", srcIndividual);
    addEvent(dc, e + "objectProperty", objectProperty);
    addEvent(dc, e + "targetIndividual", tgtIndividual);
    addEvent(changes.get(changes.size() - 1), f + "appliedAxiom", dc);

    String changeURI = createRandomURI(ontologyURL, changedOntology, "change");
    if(addition) {
      addEvent(changeURI, RDF.TYPE, f + "AddInvidivualObjectProperty");
    }
    else {
      addEvent(changeURI, RDF.TYPE, f + "RemoveInvidivualObjectProperty");
    }
    registerChange(changeURI, srcIndividual);
  }

  public List<OEvent> getEvents() {
    return this.events;
  }

  public static void exportLog(List<OEvent> events, File file)
          throws IOException {

    BufferedWriter bw = new BufferedWriter(new FileWriter(file));
    for(OEvent oe : events) {
      bw.write(oe.toString());
      bw.newLine();
    }
    bw.close();
  }

  public static void exportLog(String[] events, File file) throws IOException {

    BufferedWriter bw = new BufferedWriter(new FileWriter(file));
    for(String oe : events) {
      bw.write(oe);
      bw.newLine();
    }
    bw.close();
  }

  public static List<OEvent> importLog(File file) throws IOException {
    List<OEvent> events = new ArrayList<OEvent>();
    BufferedReader br = new BufferedReader(new FileReader(file));
    String line = br.readLine();
    while(line != null) {
      events.add(OEvent.parseEvent(line));
      line = br.readLine();
    }
    br.close();
    return events;
  }

  public void addChange(String change) {
    changes.add(change);
    addEvent("http://gate.ac.uk/;" + changedOntology + ";changeSpecification",
            f + "hasChange", change);
  }

}
